==> walletpackage-1.0.21.tar.gz: lib/src/bitcoinTransaction.dart <==
import 'dart:math';

import 'package:bitcoin_flutter/bitcoin_flutter.dart';
import 'package:walletpackage/src/rlp.dart';
import 'package:web3dart/crypto.dart';

import 'walletProxy.dart';

class BitcoinIn {
  String txHash;
  int vout;
  num value;

  BitcoinIn(this.txHash, this.vout, this.value);
}

class Btransaction {
  TransactionBuilder txb;
  NetworkType network;

  Btransaction(this.txb, this.network);
}

class BitcoinTransaction {
  static Future<String> signBitcoinTransaction(
      String privateKey, Btransaction transaction) async {
    if (transaction != null) {
      try {
        ECPair ecPair =
            ECPair.fromWIF(privateKey, network: transaction.network);
        transaction.txb.inputs.forEach((element) {
          int index = transaction.txb.inputs.indexOf(element);
          transaction.txb.sign(vin: index, keyPair: ecPair);
        });
        Transaction tx = transaction.txb.buildIncomplete();
        String hex = tx.toHex();
        return hex;
      } catch (e) {
        print('BitcoinTransaction.signBitcoinTransaction error : $e');
      }
    } else {
      print(
          'BitcoinTransaction.signBitcoinTransaction error : transaction is null.');
    }
    return null;
  }

  // 6a = 106 OP_RETURN
  // 14 = 20
  // 6f6d6e69 = omni
  static const _omni = '6a146f6d6e69';
  static final BigInt dirt = BigInt.from(546);
  static String _fillLengthWith(String s,
      {int length = 16, String char = '0'}) {
    if (s == null) return '';
    if (s.length >= length) return s.substring(0, length);
    return (char * length + s).substring(s.length);
  }

  static String _createOmniPayload(BigInt value, int contract) {
    String type = contract.toRadixString(16);
    String valueHex = value.toRadixString(16);
    String payload = _omni + _fillLengthWith(type) + _fillLengthWith(valueHex);
    return payload;
  }

  static Future<Btransaction> createBitcoinTransaction(
      String from, String to, num value, num fee,
      {String type = 'BTC', List<BitcoinIn> unspends}) async {
    if (WalletProxy.bitcoinType.keys.contains(type)) {
      CoinInfo coinInfo = WalletProxy.bitcoinType[type];
      try {
        final txb = new TransactionBuilder(network: coinInfo.network);
        BigInt coinFee = numPow2BigInt(fee, coinInfo.decimals);
        BigInt coinValue = numPow2BigInt(value, coinInfo.decimals);
        BigInt needAmount = coinValue + coinFee;
        if (unspends == null) {
          //just for test
          unspends = _checkUnspends(from, needAmount);
        }
        BigInt amount = BigInt.zero;
        unspends.forEach((element) {
          amount += numPow2BigInt(element.value, coinInfo.decimals);
          txb.addInput(element.txHash, element.vout);
        });
        BigInt change = amount - coinValue - coinFee;
//        print(amount);
//        print(coinValue);
//        print(coinFee);
//        print(change);
        if (change < BigInt.zero) {
          print(
              'BitcoinTransaction.createBitcoinTransaction error : Amount is not enough.');
        } else {
          if (change >= dirt) {
            txb.addOutput(from, change.toInt());
          }
          txb.addOutput(to, coinValue.toInt());
          return Btransaction(txb, coinInfo.network);
        }
      } catch (e) {
        print('BitcoinTransaction.createBitcoinTransaction error : $e');
      }
    }
    return null;
  }

  static Future<Btransaction> createOmniTransaction(
      String from, String to, num value, num fee,
      {String type = 'USDT', List<BitcoinIn> unspends}) async {
    if (WalletProxy.omniType.keys.contains(type)) {
      CoinInfo coinInfo = WalletProxy.omniType[type];
      try {
        final txb = new TransactionBuilder(network: coinInfo.network);
        const int decimals = 8;
        BigInt coinFee = numPow2BigInt(fee, decimals);
        ;
        BigInt needAmount = coinFee + dirt;
        if (unspends == null) {
          //just for test
          unspends = _checkUnspends(from, needAmount);
        }
        BigInt amount = BigInt.zero;
        unspends.forEach((element) {
          amount += numPow2BigInt(element.value, decimals);
          txb.addInput(element.txHash, element.vout);
        });
        BigInt change = amount - dirt - coinFee;
//        print(amount);
//        print(dirt);
//        print(coinFee);
//        print(change);
        if (change < BigInt.zero) {
          print(
              'BitcoinTransaction.createOmniTransaction Amount is not enough.');
        } else {
          if (change >= dirt) {
            txb.addOutput(from, change.toInt());
          }
          txb.addOutput(to, dirt.toInt());

          BigInt bigValue = numPow2BigInt(value, coinInfo.decimals);
          int contract = num.parse(coinInfo.address).toInt();
          String payload = _createOmniPayload(bigValue, contract);
          print(payload);

          txb.addOutput(hexToBytes(payload), 0);

          return Btransaction(txb, coinInfo.network);
        }
      } catch (e) {
        print('BitcoinTransaction.createOmniTransaction error : $e');
      }
    }
    return null;
  }

  static List<BitcoinIn> _checkUnspends(String from, BigInt amount) {
    List<BitcoinIn> ins = List<BitcoinIn>()
      ..add(BitcoinIn(
          '30db0e994544a63d3b6e2e990015022dc356d740612295a0e25bad533d923da3',
          1,
          0.0441695))
      ..add(BitcoinIn(
          '54ce8da7d37b34d764fb72c1626bb684aee49f8b9272989327b2b1e191d3f4aa',
          0,
          0.00000546));
    return ins;
  }
}

==> walletpackage-1.0.22.tar.gz: lib/src/bitcoinTransaction.dart <==
import 'dart:typed_data';

import 'package:bitcoin_flutter/bitcoin_flutter.dart';
import 'package:walletpackage/src/rlp.dart';
import 'package:web3dart/crypto.dart';
import 'package:bech32/bech32.dart';
import 'package:bs58check/bs58check.dart' as bs58check;

import 'walletProxy.dart';
import 'payments/p2sh.dart';

class BitcoinIn {
  String txHash;
  int vout;
  num value;

  BitcoinIn(this.txHash, this.vout, this.value);
}

class Btransaction {
  TransactionBuilder txb;
  NetworkType network;

  Btransaction(this.txb, this.network);
}

class BitcoinTransaction {
  static Future<String> signBitcoinTransaction(
      String privateKey, Btransaction transaction) async {
    if (transaction != null) {
      try {
        ECPair ecPair =
            ECPair.fromWIF(privateKey, network: transaction.network);
        transaction.txb.inputs.forEach((element) {
          int index = transaction.txb.inputs.indexOf(element);
          transaction.txb.sign(vin: index, keyPair: ecPair);
        });
        Transaction tx = transaction.txb.buildIncomplete();
        String hex = tx.toHex();
        return hex;
      } catch (e) {
        print('BitcoinTransaction.signBitcoinTransaction error : $e');
      }
    } else {
      print(
          'BitcoinTransaction.signBitcoinTransaction error : transaction is null.');
    }
    return null;
  }

  // 6a = 106 OP_RETURN
  // 14 = 20
  // 6f6d6e69 = omni
  static const _omni = '6a146f6d6e69';
  static final BigInt dirt = BigInt.from(546);
  static String _fillLengthWith(String s,
      {int length = 16, String char = '0'}) {
    if (s == null) return '';
    if (s.length >= length) return s.substring(0, length);
    return (char * length + s).substring(s.length);
  }

  static String _createOmniPayload(BigInt value, int contract) {
    String type = contract.toRadixString(16);
    String valueHex = value.toRadixString(16);
    String payload = _omni + _fillLengthWith(type) + _fillLengthWith(valueHex);
    return payload;
  }

  static Future<Btransaction> createBitcoinTransaction(
      String from, String to, num value, num fee,
      {String type = 'BTC', List<BitcoinIn> unspends}) async {
    if (WalletProxy.bitcoinType.keys.contains(type)) {
      CoinInfo coinInfo = WalletProxy.bitcoinType[type];
//      try {
      final txb = new TransactionBuilder(network: coinInfo.network);
      BigInt coinFee = numPow2BigInt(fee, coinInfo.decimals);
      BigInt coinValue = numPow2BigInt(value, coinInfo.decimals);
      BigInt needAmount = coinValue + coinFee;
      if (unspends == null) {
        //just for test
        unspends = _checkUnspends(from, needAmount);
      }
      BigInt amount = BigInt.zero;
      unspends.forEach((element) {
        amount += numPow2BigInt(element.value, coinInfo.decimals);
        txb.addInput(element.txHash, element.vout);
      });
      BigInt change = amount - coinValue - coinFee;
//        print(amount);
//        print(coinValue);
//        print(coinFee);
//        print(change);
      if (change < BigInt.zero) {
        print(
            'BitcoinTransaction.createBitcoinTransaction error : Amount is not enough.');
      } else {
        if (change >= dirt) {
          txb.addOutput(from, change.toInt());
        }
        txb.addOutput(to, coinValue.toInt());
        return Btransaction(txb, coinInfo.network);
      }
//      } catch (e) {
//        print('BitcoinTransaction.createBitcoinTransaction error : $e');
//      }
    }
    return null;
  }

  static Future<Btransaction> createOmniTransaction(
      String from, String to, num value, num fee,
      {String type = 'USDT', List<BitcoinIn> unspends}) async {
    if (WalletProxy.omniType.keys.contains(type)) {
      CoinInfo coinInfo = WalletProxy.omniType[type];
      try {
        final txb = new TransactionBuilder(network: coinInfo.network);
        const int decimals = 8;
        BigInt coinFee = numPow2BigInt(fee, decimals);
        BigInt needAmount = coinFee + dirt;
        if (unspends == null) {
          //just for test
          unspends = _checkUnspends(from, needAmount);
        }
        BigInt amount = BigInt.zero;
        unspends.forEach((element) {
          amount += numPow2BigInt(element.value, decimals);
          txb.addInput(element.txHash, element.vout);
        });
        BigInt change = amount - dirt - coinFee;
//        print(amount);
//        print(dirt);
//        print(coinFee);
//        print(change);
        if (change < BigInt.zero) {
          print(
              'BitcoinTransaction.createOmniTransaction Amount is not enough.');
        } else {
          if (change >= dirt) {
            txb.addOutput(from, change.toInt());
          }

          Uint8List os = addressToOutputScript(to, coinInfo.network);
          print(os);
          txb.addOutput(os, dirt.toInt());

          BigInt bigValue = numPow2BigInt(value, coinInfo.decimals);
          int contract = num.parse(coinInfo.address).toInt();
          String payload = _createOmniPayload(bigValue, contract);
          print(payload);

          txb.addOutput(hexToBytes(payload), 0);

          return Btransaction(txb, coinInfo.network);
        }
      } catch (e) {
        print('BitcoinTransaction.createOmniTransaction error : $e');
      }
    }
    return null;
  }

  static Uint8List addressToOutputScript(String address, [NetworkType nw]) {
    NetworkType network = nw ?? bitcoin;
    var decodeBase58;
    var decodeBech32;
    try {
      decodeBase58 = bs58check.decode(address);
    } catch (err) {}
    if (decodeBase58 != null) {
      if (decodeBase58[0] == network.pubKeyHash) {
        P2PKH p2pkh = new P2PKH(
            data: new PaymentData(address: address), network: network);
        return p2pkh.data.output;
      }
      if (decodeBase58[0] == network.scriptHash) {
        P2SH p2sh =
            new P2SH(data: new PaymentData(address: address), network: network);
        return p2sh.data.output;
      }
    } else {
      try {
        decodeBech32 = segwit.decode(address);
      } catch (err) {}
      if (decodeBech32 != null) {
        if (network.bech32 != decodeBech32.hrp)
          throw new ArgumentError('Invalid prefix or Network mismatch');
        if (decodeBech32.version != 0)
          throw new ArgumentError('Invalid address version');
        P2WPKH p2wpkh = new P2WPKH(
            data: new PaymentData(address: address), network: network);
        return p2wpkh.data.output;
      }
    }
    throw new ArgumentError(address + ' has no matching Script');
  }

  static List<BitcoinIn> _checkUnspends(String from, BigInt amount) {
    List<BitcoinIn> ins = List<BitcoinIn>()
      ..add(BitcoinIn(
          '30db0e994544a63d3b6e2e990015022dc356d740612295a0e25bad533d923da3',
          1,
          0.0441695))
      ..add(BitcoinIn(
          '54ce8da7d37b34d764fb72c1626bb684aee49f8b9272989327b2b1e191d3f4aa',
          0,
          0.00000546));
    return ins;
  }
}
